/***************************************************************************//**
* @file     A31L12x_hal_debug_frmwrk.h
* @brief    Contains all macro definitions and function prototypes
*           support for debug_frmwrk firmware library on A31L12x
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

#ifndef _DEBUG_FRMWRK_H_
#define _DEBUG_FRMWRK_H_

#include "A31L12x_hal_uartn.h"
#include "A31L12x_hal_libcfg.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _DEBUG_MSG

//******************************************************************************
// Constant
//******************************************************************************

#define USED_UART_DEBUG_PORT     1

#if (USED_UART_DEBUG_PORT == 0)
#define DEBUG_UART_PORT          UART0
#elif (USED_UART_DEBUG_PORT == 1)
#define DEBUG_UART_PORT          UART1
#endif

//******************************************************************************
// Macro
//******************************************************************************

#define _DBG( x )       _db_msg( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBG_( x )      _db_msg_( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBC( x )       _db_char( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBD( x )       _db_dec( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBD16( x )     _db_dec_16( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBD32( x )     _db_dec_32( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBH( x )       _db_hex( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBH16( x )     _db_hex_16( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DBH32( x )     _db_hex_32( (UARTn_Type*)DEBUG_UART_PORT, x )
#define _DG()           _db_get_char( (UARTn_Type*)DEBUG_UART_PORT )
#define _DG_( x )       _db_get_ch( (UARTn_Type*)DEBUG_UART_PORT, x )

//******************************************************************************
// Variable
//******************************************************************************

extern void ( *_db_msg )( UARTn_Type* UARTx, const void* s );
extern void ( *_db_msg_ )( UARTn_Type* UARTx, const void* s );
extern void ( *_db_char )( UARTn_Type* UARTx, uint8_t ch );
extern void ( *_db_dec )( UARTn_Type* UARTx, uint8_t decn );
extern void ( *_db_dec_16 )( UARTn_Type* UARTx, uint16_t decn );
extern void ( *_db_dec_32 )( UARTn_Type* UARTx, uint32_t decn );
extern void ( *_db_hex )( UARTn_Type* UARTx, uint8_t hexn );
extern void ( *_db_hex_16 )( UARTn_Type* UARTx, uint16_t hexn );
extern void ( *_db_hex_32 )( UARTn_Type* UARTx, uint32_t hexn );
extern uint8_t ( *_db_get_char )( UARTn_Type* UARTx );
extern uint8_t ( *_db_get_ch )( UARTn_Type* UARTx, uint8_t* ch );

//******************************************************************************
// Function
//******************************************************************************

void UARTPutChar( UARTn_Type* UARTx, uint8_t ch );
void UARTPuts( UARTn_Type* UARTx, const void* str );
void UARTPuts_( UARTn_Type* UARTx, const void* str );
void UARTPutDec( UARTn_Type* UARTx, uint8_t decnum );
void UARTPutDec16( UARTn_Type* UARTx, uint16_t decnum );
void UARTPutDec32( UARTn_Type* UARTx, uint32_t decnum );
void UARTPutHex( UARTn_Type* UARTx, uint8_t hexnum );
void UARTPutHex16( UARTn_Type* UARTx, uint16_t hexnum );
void UARTPutHex32( UARTn_Type* UARTx, uint32_t hexnum );
uint8_t UARTGetChar( UARTn_Type* UARTx );
uint8_t UARTGetCh( UARTn_Type* UARTx, uint8_t* ch );
void cprintf( const char* format, ... );
void debug_frmwrk_init( void );
uint8_t getstring( void );

#endif   /* _DEBUG_MSG */

#ifdef __cplusplus
}
#endif

#endif   /* _DEBUG_FRMWRK_H_ */

