/***************************************************************************//**
* @file     A31L12x_hal_sculv.h
* @brief    Contains all macro definitions and function prototypes
*           support for sculv firmware library on A31L12x
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

#ifndef _SCULV_H_
#define _SCULV_H_

#include "A31L12x.h"
#include "A31L12x_hal_aa_types.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
// Constant
//******************************************************************************

// LVI Enable/Disable Control
/*
#define LVI_DIS                  (0x00uL << 7)
#define LVI_EN                   (0x01uL << 7)
*/
#define LVIEN_Disable            (SCULV_LVICR_LVIEN_Disable << SCULV_LVICR_LVIEN_Pos)
#define LVIEN_Enable             (SCULV_LVICR_LVIEN_Enable  << SCULV_LVICR_LVIEN_Pos)

// LVI Interrupt Enable/Disable Control
/*
#define LVI_INT_DIS              (0x00uL << 5)
#define LVI_INT_EN               (0x01uL << 5)
*/
#define LVINTEN_Disable          (SCULV_LVICR_LVINTEN_Disable << SCULV_LVICR_LVINTEN_Pos)
#define LVINTEN_Enable           (SCULV_LVICR_LVINTEN_Enable  << SCULV_LVICR_LVINTEN_Pos)

// LVI Voltage Selection Control
/*
#define LV_1DOT88                (0x03uL)
#define LV_2DOT00                (0x04uL)
#define LV_2DOT13                (0x05uL)
#define LV_2DOT28                (0x06uL)
#define LV_2DOT46                (0x07uL)
#define LV_2DOT67                (0x08uL)
#define LV_3DOT04                (0x09uL)
#define LV_3DOT20                (0x0AuL)
#define LV_3DOT55                (0x0BuL)
#define LV_3DOT75                (0x0CuL)
#define LV_3DOT99                (0x0DuL)
#define LV_4DOT25                (0x0EuL)
#define LV_4DOT55                (0x0FuL)
*/
#define LVIVS_1p75V0             (SCULV_LVICR_LVIVS_1p75V0 << SCULV_LVICR_LVIVS_Pos)     // 1.75V                                                             */
#define LVIVS_1p75V1             (SCULV_LVICR_LVIVS_1p75V1 << SCULV_LVICR_LVIVS_Pos)     // 1.75V                                                             */
#define LVIVS_1p90V              (SCULV_LVICR_LVIVS_1p90V  << SCULV_LVICR_LVIVS_Pos)     // 1.90V                                                              */
#define LVIVS_2p05V              (SCULV_LVICR_LVIVS_2p05V  << SCULV_LVICR_LVIVS_Pos)     // 2.05V                                                              */
#define LVIVS_2p20V              (SCULV_LVICR_LVIVS_2p20V  << SCULV_LVICR_LVIVS_Pos)     // 2.20V                                                              */
#define LVIVS_2p35V              (SCULV_LVICR_LVIVS_2p35V  << SCULV_LVICR_LVIVS_Pos)     // 2.35V                                                              */
#define LVIVS_2p50V              (SCULV_LVICR_LVIVS_2p50V  << SCULV_LVICR_LVIVS_Pos)     // 2.50V                                                              */
#define LVIVS_2p65V              (SCULV_LVICR_LVIVS_2p65V  << SCULV_LVICR_LVIVS_Pos)     // 2.65V                                                              */

//******************************************************************************
// Macro
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Enable/Disable LVR
 * @details       This macro disables/enables LVR. If any value is written, the LVR register won't be changed until to POR.
 *//*-------------------------------------------------------------------------*/
#define SCULV_DisLVR()           (SCULV->LVRCR_b.LVREN = 0x55)
#define SCULV_EnLVR()            (SCULV->LVRCR_b.LVREN = 0x8A)

/*-------------------------------------------------------------------------*//**
 * @brief         Enable/Disable LVI Interrupt
 * @details       This macro sets the LVI interrupt enable bit.
 *//*-------------------------------------------------------------------------*/
#define SCULV_EnLviInt()         (SCULV->LVICR_b.LVINTEN = 1)
#define SCULV_DisLviInt()        (SCULV->LVICR_b.LVINTEN = 0)

/*-------------------------------------------------------------------------*//**
 * @brief         Get/Clear LVI Interrupt Flag
 * @details       This macro gets/clears the LVI interrupt flag.
 *//*-------------------------------------------------------------------------*/
#define SCULV_GetLviFlag()       (SCULV->LVICR_b.LVIFLAG)
#define SCULV_ClrLviFlag()       (SCULV->LVICR_b.LVIFLAG = 1)

//******************************************************************************
// Function
//******************************************************************************

void HAL_LVI_Init( uint32_t u32LviEnBit, uint32_t u32LviIntEnBit, uint32_t u32LviLevel );

#ifdef __cplusplus
}
#endif

#endif   /* _SCULV_H_ */

