/***************************************************************************//**
* @file     A31L12x_hal_pmu.c
* @brief    Contains all functions support for pmu firmware library on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

/* Includes ----------------------------------------------------------------- */
//******************************************************************************
// Include
//******************************************************************************

#include "A31L12x_hal_pmu.h"

/* Public Functions --------------------------------------------------------- */
//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Enable all power off function
 * @return        None
 * @details       This function powers off all system and peripheral except always-on region on deep sleep mode.
 *//*-------------------------------------------------------------------------*/
void HAL_PMU_Enable_AllPowerOff( void )
{
   PMU->PWRCR = PMU->PWRCR
                | ( PMU_PWRCR_ALLPWR_Off   << PMU_PWRCR_ALLPWR_Pos )
                | ( PMU_PWRCR_WTIDKY_Value << PMU_PWRCR_WTIDKY_Pos )
                ;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Enable flash power off
 * @return        None
 * @details       This function powers off the flash.
 *//*-------------------------------------------------------------------------*/
void HAL_PMU_Enable_FlashPowerOff( void )
{
   PMU->PWRCR = PMU->PWRCR
                | ( PMU_PWRCR_FLASHPWR_Off << PMU_PWRCR_FLASHPWR_Pos )
                | ( PMU_PWRCR_WTIDKY_Value << PMU_PWRCR_WTIDKY_Pos )
                ;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Disable flash power off
 * @return        None
 * @details       This function powers on the flash.
 *//*-------------------------------------------------------------------------*/
void HAL_PMU_Disable_FlashPowerOff( void )
{
   PMU->PWRCR = PMU->PWRCR
                & ~( PMU_PWRCR_FLASHPWR_Off << PMU_PWRCR_FLASHPWR_Pos )
                | ( PMU_PWRCR_WTIDKY_Value  << PMU_PWRCR_WTIDKY_Pos )
                ;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Enable the sram retention power function
 * @return        None
 * @details       This function selects the retention power for sram on sleep and deep sleep mode.
 *//*-------------------------------------------------------------------------*/
void HAL_PMU_Enable_SRAMRetentionPower( void )
{
   PMU->PWRCR = PMU->PWRCR
                | ( PMU_PWRCR_SRAMRTPWR_RetentionPower << PMU_PWRCR_SRAMRTPWR_Pos )
                | ( PMU_PWRCR_WTIDKY_Value             << PMU_PWRCR_WTIDKY_Pos )
                ;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Disable the sram retention power function
 * @return        None
 * @details       This function selects the normal power for sram on sleep and deep sleep mode.
 *//*-------------------------------------------------------------------------*/
void HAL_PMU_Disable_SRAMRetentionPower( void )
{
   PMU->PWRCR = PMU->PWRCR
                & ~( PMU_PWRCR_SRAMRTPWR_RetentionPower << PMU_PWRCR_SRAMRTPWR_Pos )
                | ( PMU_PWRCR_WTIDKY_Value              << PMU_PWRCR_WTIDKY_Pos )
                ;
}

