/***************************************************************************//**
* @file     A31L12x_hal_pwr.c
* @brief    Contains all functions support for pwr firmware library on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements,
*modifications, and improvements to ABOV products and/or to this document at any time without notice.
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders.
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and
*ABOV assumes no liability for application assistance or the design of purchasers' products. No license,
*express or implied, to any intellectual property rights is granted by ABOV herein.
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications.
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners.
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
*//****************************************************************************/

/* Includes ----------------------------------------------------------------- */
//******************************************************************************
// Include
//******************************************************************************

#include "A31L12x_hal_pwr.h"

/* Public Functions --------------------------------------------------------- */
//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         Enter Sleep mode with co-operated instruction by the Cortex-M0+.
 * @param         None
 * @return        None
 * @note          The 6 NOP Instruction should follow immediately after WFI or WFE Instruction.
 *//*-------------------------------------------------------------------------*/
void HAL_PWR_EnterSleepMode( void )
{
   // Sleep Mode, clear SLEEPDEEP bit
   SCB->SCR = 0;

   // Sleep Mode
   WFI();
   NOP();
   NOP();
   NOP();
   NOP();
   NOP();
   NOP();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Enter Deep Sleep mode with co-operated instruction by the Cortex-M0+.
 * @param         None
 * @return        None
 * @note          The 6 NOP Instruction should follow immediately after WFI or WFE Instruction.
 *//*-------------------------------------------------------------------------*/
void HAL_PWR_EnterPowerDownMode( void )
{
   // Deep-Sleep Mode, set SLEEPDEEP bit
   SCB->SCR = 0x4;

   // Deep Sleep Mode
   WFI();
   NOP();
   NOP();
   NOP();
   NOP();
   NOP();
   NOP();
}

