/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void ADC_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void ADC_Configure( void );
void ADC_ContinuousModeRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "ADC_ContinuousMode Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: ADC \n\r"
                        "ADC Input Pin:     PA1(AN1) PA4(AN4) PA7(AN7) PA8(AN8) \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    fflag;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_IRQHandler_IT( void )
{
   if( ADCInt_GetEOSFg() )
   {
      fflag = 1;
      ADCInt_ClrEOSFg();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_Configure( void )
{
   ADC_CFG_Type      ADC_Config;

   // ADC configuration
   {
      // adc input pin configuration
      {
         // print
         _DBG( "PA1 PA4 PA7 PA8 was configured as a AN1 AN4 AN7 AN8. \n\r\n\r" );

         // configure PA1 as a AN1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, 4 );

         // configure PA4 as a AN4
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 4, 4 );

         // configure PA7 as a AN7
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 7, 4 );

         // configure PA8 as a AN8
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 8, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 8, 4 );
      }

      // adc configuration
      {
         ADC_Config.TrgSel = ADC_CR_TRIG_ADST;
         ADC_Config.ExtTrgPolSel = ADC_CR_ETRGP_Disable;
         ADC_Config.ConvModSel = ADC_CR_MDSEL_Continuous;
         ADC_Config.InClkDiv = 1;         // 32M/2 = 16M
         ADC_Config.SamplingClk = 2;      // Sample&Hold: n+2 = 4 cycles
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
         HAL_ADC_ConfigInterrupt( ADC, ADC_INTCFG_EOS, ENABLE );
         HAL_ADC_ClearStatus( ADC, ADC_INT_STATUS_ALL );
      }

      // unmask Interrupt
      NVIC_SetPriority( ADC_IRQn, 3 );    // Enable Interrupt for ADC channel
      NVIC_EnableIRQ( ADC_IRQn );         // ADC interrupt unmask
      HAL_INT_EInt_MaskDisable( MSK_ADC );

      // select channel
      HAL_ADC_OvSamplingConfig( ADC, ADC_8_OV_SAMPLING, ADC_OVSCR_OVSHT_ShiftRight3Bit );     // Oversampling ratio, Data shift right
      HAL_ADC_ChannelSel( ADC, ADC_AN1 | ADC_AN4 | ADC_AN7 | ADC_AN8 );    // select channels

      // ready adc
      HAL_ADC_RdyCmd( ADC );     // Ready for conversion

      // start adc
      HAL_ADC_Start( ADC );      // Start
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_ContinuousModeRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_ContinuousModeRun( void )
{
   uint16_t       adc_result;
   uint32_t       tmp;

   /* Infinite loop */
   while( 1 )
   {
      fflag = 0;

      //------------------
      while( !ADCInt_GetEOCFg() ) {}
      if( ADC_GetLastCh() != 1 )
      {
         adc_result = HAL_ADC_GetData( ADC );
         _DBG( "Error" );
         _DBG( " " );
      }
      else
      {
         adc_result = HAL_ADC_GetData( ADC );

         _DBG( "AN1 = " );
         _DBH16( adc_result );
         _DBG( " " );
      }

      //------------------
      while( !ADCInt_GetEOCFg() ) {}
      if( ADC_GetLastCh() != 4 )
      {
         adc_result = HAL_ADC_GetData( ADC );
         _DBG( "Error" );
         _DBG( " " );
      }
      else
      {
         adc_result = HAL_ADC_GetData( ADC );

         _DBG( "AN4 = " );
         _DBH16( adc_result );
         _DBG( " " );
      }

      //------------------
      while( !ADCInt_GetEOCFg() ) {}
      if( ADC_GetLastCh() != 7 )
      {
         adc_result = HAL_ADC_GetData( ADC );
         _DBG( "Error" );
         _DBG( " " );
      }
      else
      {
         adc_result = HAL_ADC_GetData( ADC );

         _DBG( "AN7 = " );
         _DBH16( adc_result );
         _DBG( " " );
      }

      //------------------
      while( !ADCInt_GetEOCFg() ) {}
      if( ADC_GetLastCh() != 8 )
      {
         adc_result = HAL_ADC_GetData( ADC );
         _DBG( "Error" );
         _DBG( " " );
      }
      else
      {
         adc_result = HAL_ADC_GetData( ADC );

         _DBG( "AN8 = " );
         _DBH16( adc_result );
         _DBG( " " );
      }

      _DBG( "\n\r" );
      while( fflag == 0 ) {}
      for( tmp = 0; tmp < 1000000; tmp++ );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* ADC_Configure */
   ADC_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* ADC_ContinuousModeRun */
   ADC_ContinuousModeRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

