/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void CMPn_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void CMP0_Configure( void );
void CMPn_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "CMPn_Test Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:             CMP0 \n\r"
                        "GPIO Output Pin:               PB0 ~ PB3 \n\r"
                        "Comparator Positive Input Pin: PA1(CP0P0) \n\r"
                        "Comparator Negative Input Pin: Internal Reference \n\r"
                        "Comparator Output Pin:         PC3(CP0OUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         CMPn_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CMPn_IRQHandler_IT( void )
{
   uint32_t          status;
   static uint8_t    alternate = 1;    // toggle flag

   // if interrupt was issued by CMP0
   status = HAL_COMP_GetStatus( ( CMPn_Type* )CMP0 );
   if( ( CMP0->CR_b.CMPnEN == CMPn_CR_CMPnEN_Enable ) && ( status & CMPn_SR_CMPnIFLAG_Msk ) )
   {
      // clear interrupt flag
      HAL_COMP_ClearStatus( ( CMPn_Type* )CMP0, CMPn_SR_CMPnIFLAG_Msk );

      // toggle PB0 ~ PB3
      if( alternate == 1 )
      {
         // clear flag
         alternate = 0;

         // output high
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
      else
      {
         // set flag
         alternate = 1;

         // output low
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
   }

   // if interrupt was issued by CMP1
   status = HAL_COMP_GetStatus( ( CMPn_Type* )CMP1 );
   if( ( CMP1->CR_b.CMPnEN == CMPn_CR_CMPnEN_Enable ) && ( status & CMPn_SR_CMPnIFLAG_Msk ) )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         Delay function
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         CMP0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CMP0_Configure( void )
{
   // CMP0 configuration
   {
      // comparator positive input pin configuration
      {
         // print
         _DBG( "PA1 was configured as a CP0P0. \n\r\n\r" );

         // configure PA1 as a CP0P0
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );      // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, AFSRx_AF5 );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 1, PUPDx_EnablePU );
      }

      // comparator output pin configuration
      {
         // print
         _DBG( "PC3 was configured as a CP0OUT. \n\r\n\r" );

         // configure PC3 as a CP0OUT
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 3, ALTERN_FUNC );      // PC3            0: ----        1: ----        2: ----        3: MISO1       4: SC0RST      5: SC0RXD      6: CP0OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 3, AFSRx_AF6 );
      }

      // comparator configuration
      {
         HAL_COMP_Init( ( CMPn_Type* )CMP0 );
         HAL_COMP_ConfigPositiveInput( ( CMPn_Type* )CMP0, CMPnPOS_CPnP0 );
         HAL_COMP_ConfigNegativeInput( ( CMPn_Type* )CMP0, CMPnNEG_IntRef );
         HAL_COMP_ConfigIntRef( ( CMPn_Type* )CMP0, REFnEN_Enable, CMPnREF_6over11VDD );
         HAL_COMP_ConfigHysteresis( ( CMPn_Type* )CMP0, HYSnEN_Enable );
         HAL_COMP_ConfigSpeed( ( CMPn_Type* )CMP0, CMPnSPD_Slow );
         HAL_COMP_ConfigNoiseFilter( ( CMPn_Type* )CMP0, NFCKn_PCLK1 );
         HAL_COMP_ConfigInterruptPolarity( ( CMPn_Type* )CMP0, CMPnPOL_BothEdge );
         HAL_COMP_Enable( ( CMPn_Type* )CMP0 );
      }

      // unmask Interrupt
      NVIC_SetPriority( CMPn_IRQn, 3 );
      NVIC_EnableIRQ( CMPn_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_CMPn );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         CMPn_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void CMPn_InterruptRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
      // get Output Status
      if( CMP0->SR_b.CMPnST == CMPn_SR_CMPnST_Low )
      {
         _DBG( "Output Status is Low. \n\r" );
      }
      else
      {
         _DBG( "Output Status is High. \n\r" );
      }

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* CMP0_Configure */
   CMP0_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* CMPn_InterruptRun */
   CMPn_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

