/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void DMAC0_IRQHandler_IT( void );
void DMAC1_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SCIn_Configure( void );
void DMAC0_Configure( void );
void DMAC1_Configure( void );
void DMAC2_Configure( void );
void DMAC3_Configure( void );
void DMACn_SCn_SCIRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// buffer size
#define BUF_SIZE     4

//******************************************************************************
// Type
//******************************************************************************

//==============================================================================
// Enumeration
//==============================================================================

// Card State Definition
typedef enum
{
   CARD_NONE = 0,
   CARD_START,
   CARD_ACTIVATION,
   CARD_WAIT_ATR,
   CARD_DEACTIVATION,
   CARD_INTERFACE
} CARD_STATE_Type;

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "DMACn_SCn_SCI Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: DMAC0 / DMAC1 / SCn \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "  - Monitoring: SCI - 5434-8-E-1 \n\r"
                        "Target Peripheral: SC0 \n\r"
                        "Counterpart:       SC1: Only Reflect \n\r"
                        "************************************************\n\r\n\r";

// transfer complete flag
volatile Bool     TransferComplete0 = FALSE;
volatile Bool     TransferComplete1 = FALSE;
//volatile Bool     TransferComplete2 = FALSE;
//volatile Bool     TransferComplete3 = FALSE;

// transfer error flag
volatile Bool     TransferError0 = FALSE;
volatile Bool     TransferError1 = FALSE;
//volatile Bool     TransferError2 = FALSE;
//volatile Bool     TransferError3 = FALSE;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC0->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete0 = TRUE;

      // reset TRCIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC0->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError0 = TRUE;

      // reset TRERIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC1->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete1 = TRUE;

      // reset TRCIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC1->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError1 = TRUE;

      // reset TRERIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SCIn_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SCIn_Configure( void )
{
   SCn_CFG_Type   SCnConfigStruct;

   // SCI0 ~ SCI1 configuration
   {
      // sci IN/PWR/CLK/RST/DATA pin configuration
      {
         // print
         _DBG( "PC[4:0] was configured as a SCI mode pins of SC0. \n\r" );
         _DBG( "PD2: SC1DATA, Connect to SC0DATA pin. \n\r" );
         _DBG( "PD4: Output To active the SC0 block. Connect to SC0IN pin. \n\r\n\r" );

         // configure PC0 as a SC0IN
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );      // PC0            0: CLKO        1: ----        2: ----        3: ----        4: SC0IN       5: ----        6: ----        7: COM0
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF4 );   // PC10           0: ----        1: ----        2: TXD1        3: ----        4: SC0IN       5: ----        6: ----        7: COM6/SEG27
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PC, 0, DBCLK_HCLK64 );

         // configure PC1 as a SC0PWR
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 1, ALTERN_FUNC );      // PC1            0: CLKO        1: ----        2: TXD0        3: ----        4: SC0PWR      5: ----        6: ----        7: COM1
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 1, AFSRx_AF4 );   // PC9            0: ----        1: ----        2: LPRXD       3: SS 10       4: SC0PWR      5: ----        6: ----        7: COM5/SEG26

         // configure PC2 as a SC0CLK
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 2, ALTERN_FUNC );      // PC2            0: ----        1: ----        2: RXD0        3: ----        4: SC0CLK      5: ----        6: ----        7: COM2
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 2, AFSRx_AF4 );   // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25

         // configure PC3 as a SC0RST
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 3, ALTERN_FUNC );      // PC3            0: ----        1: ----        2: ----        3: MISO1       4: SC0RST      5: SC0RXD      6: CP0OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 3, AFSRx_AF4 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24

         // configure PC4 as a SC0DATA
         PC->PC_TYP_b.TYP4 = Pn_TYP_TYP4_OpenDrain;
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 4, ALTERN_FUNC );      // PC4            0: ----        1: ----        2: ----        3: MOSI1       4: SC0DATA     5: SC0TXD      6: CP1OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 4, AFSRx_AF4 );   // PC6   SWCLK    0: SWCLK       1: ----        2: TXD10       3: MOSI10      4: SC0DATA     5: SC0TXD      6: ----        7: ----

         // configure PD4 as a output
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, PUSH_PULL_OUTPUT );

         // configure PD2 as a SC1DATA
         PD->PD_TYP_b.TYP2 = Pn_TYP_TYP2_OpenDrain;
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );      // PD2            0: T43OUTB     1: EC43        2: EC50        3: MOSI1       4: SC1DATA     5: SC1TXD      6: CP1P3       7: SEG31
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF4 );
      }

      // sci configuration
      {
         // get default SCnConfigStruct for SCI: 8-Y-1
         HAL_SC_ConfigStructInit( &SCnConfigStruct, SCn_SCI_MODE, 5434 );

         // init SCn
         SCnConfigStruct.Baud_rate_Compensation = 0;
         HAL_SC_Init( ( SCn_Type* )SC0, &SCnConfigStruct );
         HAL_SC_Init( ( SCn_Type* )SC1, &SCnConfigStruct );

         // enable SCn
         HAL_SC_Enable( ( SCn_Type* )SC0, ENABLE );

         HAL_SC_Enable( ( SCn_Type* )SC1, ENABLE );
         HAL_SC_AutoAct( ( SCn_Type* )SC1 );
         delay();
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_Configure( void )
{
   // DMAC0 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC0, PERSEL_SC0Tx, DIR_MemToPeri, SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC0->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC0->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC0_IRQn, 3 );
      NVIC_EnableIRQ( DMAC0_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC0 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_Configure( void )
{
   // DMAC1 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC1, PERSEL_SC0Rx, DIR_PeriToMem,  SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC1->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC1->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC1_IRQn, 4 );
      NVIC_EnableIRQ( DMAC1_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC2_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC2_Configure( void )
{
   // DMAC2 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC2, PERSEL_SC1Tx, DIR_MemToPeri,  SIZE_8bit, ERFGSTP_Enable );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC3_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC3_Configure( void )
{
   // DMAC3 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC3, PERSEL_SC1Rx, DIR_PeriToMem,  SIZE_8bit, ERFGSTP_Enable );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMACn_SCn_SCIRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMACn_SCn_SCIRun( void )
{
   uint32_t             len_pc = 0, intsrc;
   uint8_t              buffer_pc[BUF_SIZE], buffer_sc0[BUF_SIZE], buffer_sc1[BUF_SIZE];
   CARD_STATE_Type      card_state = CARD_NONE;

   /* Infinite loop */
   while( 1 )
   {
      switch( card_state )
      {
         case CARD_NONE:
            {
               // to set high level at SC0IN Pin for test
               GPIO_SetBit4( PD );
               _DBG( "The PD[4] outputs high level to test SCI mode. \n\r" );
               delay();
               card_state = CARD_START;
               break;
            }
         case CARD_START:
            {
               if( HAL_SC_GetSCnINST( ( SCn_Type* )SC0 ) )
               {
                  HAL_SC_AutoConvDet( ( SCn_Type* )SC0 );
                  HAL_SC_AutoAct( ( SCn_Type* )SC0 );
                  card_state = CARD_ACTIVATION;
               }
               break;
            }
         case CARD_ACTIVATION:
            {
               intsrc = HAL_SC_GetStatus( ( SCn_Type* )SC0 );
               intsrc &= SCn_IFSR_BITMASK;

               // if Reset Assertion Interrupt
               if( ( intsrc & SCn_IFSR_RSTA ) == SCn_IFSR_RSTA )
               {
                  _DBG( "The value of SC0->IFSR is " );
                  _DBH32( intsrc );
                  _DBG( ". Reset assertion finish. \n\r" );
                  HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_RSTA );
                  card_state = CARD_WAIT_ATR;
                  break;
               }

               // if Sequence Error Interrupt
               if( ( intsrc & SCn_IFSR_SER ) == SCn_IFSR_SER )
               {
                  _DBG( "The value of SC0->IFSR is " );
                  _DBH32( intsrc );
                  _DBG( ". Sequence error occur. \n\r" );
                  HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_SER );
                  HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
                  HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
                  card_state = CARD_DEACTIVATION;
               }
               break;
            }
         case CARD_WAIT_ATR:
            {
               buffer_sc1[0] = DIRECT_CONV_DATA;
               HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, DISABLE );
               HAL_SC_Transmit( ( SCn_Type* )SC1, buffer_sc1, 1, BLOCKING );

               intsrc = HAL_SC_GetStatus( ( SCn_Type* )SC0 );
               intsrc &= SCn_IFSR_BITMASK;

               // if Sequence or Convention Error Interrupt
               if( ( ( intsrc & SCn_IFSR_SER ) == SCn_IFSR_SER ) || ( ( intsrc & SCn_IFSR_CONER ) == SCn_IFSR_CONER ) )
               {
                  _DBG( "The value of SC0->IFSR is " );
                  _DBH32( intsrc );
                  _DBG( ". Sequence or convention error occur. \n\r" );
                  HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
                  HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
                  card_state = CARD_DEACTIVATION;
                  break;
               }

               // if Sequence or Convention End Interrupt
               if( ( ( intsrc & SCn_IFSR_SED ) == SCn_IFSR_SED ) && ( ( intsrc & SCn_IFSR_CONED ) == SCn_IFSR_CONED ) )
               {
                  _DBG( "The value of SC0->IFSR is " );
                  _DBH32( intsrc );
                  _DBG( ". Sequence and convention finish. \n\r" );
                  HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_ALL_EXCEPT_TXC );
                  HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, ENABLE );
                  card_state = CARD_INTERFACE;
                  break;
               }

               _DBG( "The value of SC0->IFSR is " );
               _DBH32( intsrc );
               _DBG( ". Sequence and convention finish. \n\r" );
               HAL_SC_SetT34DR( ( SCn_Type* )SC0, 150, 150 );
               HAL_SC_AutoDeAct( ( SCn_Type* )SC0 );
               card_state = CARD_DEACTIVATION;
               break;
            }
         case CARD_DEACTIVATION:
            {
               GPIO_ClrBit4( PD );
               _DBG( "The PD[4] outputs low level to finish SCI mode. \n\r" );
               delay();
               delay();
               HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_ALL_EXCEPT_TXC );
               HAL_SC_SetT34DR( ( SCn_Type* )SC0, 10000 - 1, 40000 - 1 );
               card_state = CARD_NONE;
               break;
            }
         case CARD_INTERFACE:
            {
               // PC -> UART1
               if( ( UART1->LSR & UARTn_LSR_RDR ) )
               {
                  buffer_pc[len_pc++] = HAL_UART_ReceiveByte( ( UARTn_Type* )UART1 );

                  // if rx transfer complete (receive data from PC)
                  if( len_pc >= BUF_SIZE )
                  {
                     len_pc = 0;

                     // SC0 Tx -> SC1 Rx
                     HAL_SC_ControlConfig( ( SCn_Type* )SC0, SCn_CR1_RXE, DISABLE );
                     HAL_DMAC_Setup( ( DMACn_Type* )DMAC3, ( uint32_t )buffer_sc1, BUF_SIZE );
                     HAL_DMAC_Setup( ( DMACn_Type* )DMAC0, ( uint32_t )buffer_pc, BUF_SIZE );
                  }
               }

               // if SC0 Tx finish, start SC1 Tx -> SC0 Rx
               if( ( TransferComplete0 == TRUE ) && ( SC0->IFSR_b.TXCIFLAGn ) )   // Tx through DMA is completed when the TXC flag is set to '1' after TransferComplete0 == TRUE.
               {
                  // clear flag
                  TransferComplete0 = FALSE;

                  // SC1 Tx -> SC0 Rx
                  HAL_SC_ControlConfig( ( SCn_Type* )SC0, SCn_CR1_RXE, ENABLE );
                  HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, DISABLE );
                  HAL_DMAC_Setup( ( DMACn_Type* )DMAC1, ( uint32_t )buffer_sc0, BUF_SIZE );
                  HAL_DMAC_Setup( ( DMACn_Type* )DMAC2, ( uint32_t )buffer_sc1, BUF_SIZE );
               }
               if( ( HAL_SC_GetStatus( ( SCn_Type* )SC1 ) & SCn_IFSR_DOR ) == SCn_IFSR_DOR )
               {
                  HAL_SC_ClearStatus( ( SCn_Type* )SC1, SCn_STATUS_DOR );
               }

               // if SC0 Rx finish, start SC0 Rx -> PC
               if( ( TransferComplete1 == TRUE )  && ( SC1->IFSR_b.TXCIFLAGn ) )  // Tx through DMA is completed when the TXC flag is set to '1' after TransferComplete1 == TRUE.
               {
                  // clear flag
                  TransferComplete1 = FALSE;

                  // SC0 Rx -> PC
                  HAL_SC_ControlConfig( ( SCn_Type* )SC1, SCn_CR1_RXE, ENABLE );
                  HAL_UART_Transmit( ( UARTn_Type* )UART1, buffer_sc0, 4, BLOCKING );
               }
               if( ( HAL_SC_GetStatus( ( SCn_Type* )SC0 ) & SCn_IFSR_DOR ) == SCn_IFSR_DOR )
               {
                  HAL_SC_ClearStatus( ( SCn_Type* )SC0, SCn_STATUS_DOR );
               }
               break;
            }
         default:
            break;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SCIn_Configure */
   SCIn_Configure();

   /* DMAC0_Configure */
   DMAC0_Configure();

   /* DMAC1_Configure */
   DMAC1_Configure();

   /* DMAC2_Configure */
   DMAC2_Configure();

   /* DMAC3_Configure */
   DMAC3_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DMACn_SCn_SCIRun */
   DMACn_SCn_SCIRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

