/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void DMAC0_IRQHandler_IT( void );
void DMAC1_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SPI1_Configure( void );
void DMAC0_Configure( void );
void DMAC1_Configure( void );
void DMACn_SPInRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// buffer size
#define BUF_SIZE     4

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "DMACn_SPIn Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:   DMAC0 / DMAC1 / SPI1 Master \n\r"
                        "Counterpart:         Self (Loopback) \n\r"
                        "Master Tx Data:      'A' ~ 'Z' \n\r"
                        "SPI Master MOSI Pin: PA7(MOSI1)       \n\r"
                        "SPI Master MISO Pin: PA6(MISO1)       \n\r"
                        "SPI Master SCLK Pin: PA5(SCK1)        \n\r"
                        "SPI Master nSS Pin:  PA4(Output Mode) \n\r"
                        "Prerequisite:        Connect PA7(MOSI1) and PA6(MISO1) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

// transfer complete flag
volatile Bool     TransferComplete0 = FALSE;
volatile Bool     TransferComplete1 = FALSE;

// transfer error flag
volatile Bool     TransferError0 = FALSE;
volatile Bool     TransferError1 = FALSE;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI_SS_control
 * @param[in]     SPIx
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI_SS_control( SPIn_Type* SPIx, char ch )
{
#if 1    // supported
   if( SPIx == ( SPIn_Type* )SPI1 )
   {
      if( ch == 0 )
      {
         PA->OUTDR_b.OUTDR1 = 0;    // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
      }                             // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
      else
      {
         PA->OUTDR_b.OUTDR1 = 1;    // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
      }                             // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
   }
#endif

#if 0    // not supported
   if( SPIx == ( SPIn_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
#endif

#if 0    // not supported
   if( SPIx == ( SPIn_Type* )USART12 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR15 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR15 = 1;
      }
   }
#endif

#if 0    // not supported
   if( SPIx == ( SPIn_Type* )USART13 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR11 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR11 = 1;
      }
   }
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC0->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete0 = TRUE;

      // reset TRCIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC0->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError0 = TRUE;

      // reset TRERIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC1->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete1 = TRUE;

      // reset TRCIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC1->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError1 = TRUE;

      // reset TRERIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI1_Configure( void )
{
   SPIn_CFG_Type     SPIn_Config;

   // SPI1 configuration
   {
      // spi MOSI/MISO/SCLK/nSS pin configuration
      {
         // print
         _DBG( "PA7 was configured as a MOSI1. \n\r" );
         _DBG( "PA6 was configured as a MISO1. \n\r" );
         _DBG( "PA5 was configured as a SCK1. \n\r" );
#if 1    // if Msater Mode
         _DBG( "PA4 was configured as a Output Mode. \n\r\n\r" );
#else    // if Slave Mode
         _DBG( "PA4 was configured as a SS1. \n\r\n\r" );
#endif

         // configure PA7 as a MOSI1                              // PA7            0: T43OUTB     1: EC43        2: ----        3: MOSI1       4: AN7         5: CP1P1       6: CP1OUT      7: SEG15
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 7, ALTERN_FUNC );      // PC4            0: ----        1: ----        2: ----        3: MOSI1       4: SC0DATA     5: SC0TXD      6: CP1OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 7, AFSRx_AF3 );   // PD2            0: T43OUTB     1: EC43        2: EC50        3: MOSI1       4: SC1DATA     5: SC1TXD      6: CP1P3       7: SEG31

         // configure PA6 as a MISO1                              // PA6            0: T43OUTA     1: T43INP      2: ----        3: MISO1       4: AN6         5: ----        6: CP0OUT      7: SEG16
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 6, ALTERN_FUNC );      // PC3            0: ----        1: ----        2: ----        3: MISO1       4: SC0RST      5: SC0RXD      6: CP0OUT      7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 6, AFSRx_AF3 );   // PD1            0: T43OUTA     1: T43INP      2: ----        3: MISO1       4: SC1RST      5: SC1RXD      6: CP1P2       7: SEG30

         // configure PA5 as a SCK1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 5, ALTERN_FUNC );      // PA5            0: T40OUTA     1: T40INP      2: ----        3: SCK1        4: AN5         5: CP0N2       6: CP1N2       7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 5, AFSRx_AF3 );   // PD0            0: T40OUTB     1: EC40        2: ----        3: SCK1        4: SC1CLK      5: CP1N3       6: ----        7: SEG29

#if 1    // if Msater Mode
         // configure PA4 as a Output Mode                        // PA4            0: ----        1: T43INP      2: SS1         3: SCK10       4: AN4         5: CP0N1       6: CP1N1       7: ----
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, PUSH_PULL_OUTPUT ); // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24
#else    // if Slave Mode
         // configure PA4 as a SS1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, ALTERN_FUNC );      // PA4            0: ----        1: T43INP      2: SS1         3: SCK10       4: AN4         5: CP0N1       6: CP1N1       7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 4, AFSRx_AF2 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24
#endif
      }

      // spi configuration
      {
         // get default configuration for spi: 38400
         SPIn_ConfigStructInit( &SPIn_Config );

         // init spin
         HAL_SPIn_Init( ( SPIn_Type* )SPI1, &SPIn_Config );

#if 1    // if Msater Mode
         // enable master
         SPIn_DataControlConfig( ( SPIn_Type* )SPI1, SPIn_CONTROL_SPInMS, ENABLE );

         // nss = 1
         SPI_SS_control( ( SPIn_Type* )SPI1, 1 );
#else    // if Slave Mode
         // disable master
         SPIn_DataControlConfig( ( SPIn_Type* )SPI1, SPIn_CONTROL_SPInMS, DISABLE );

         // enable slave nss pin function
         SPIn_DataControlConfig( ( SPIn_Type* )SPI1, SPIn_CONTROL_SSnEN, ENABLE );
#endif

         // enable SPIn
         SPIn_Enable( ( SPIn_Type* )SPI1, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_Configure( void )
{
   // DMAC0 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC0, PERSEL_SPI1Tx, DIR_MemToPeri, SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC0->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC0->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC0_IRQn, 3 );
      NVIC_EnableIRQ( DMAC0_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC0 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_Configure( void )
{
   // DMAC1 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC1, PERSEL_SPI1Rx, DIR_PeriToMem,  SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC1->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC1->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC1_IRQn, 3 );
      NVIC_EnableIRQ( DMAC1_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMACn_SPInRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMACn_SPInRun( void )
{
   uint8_t     i;
   uint8_t     mtxch = 'A';         // master tx character
   uint8_t     mtxbuf[BUF_SIZE];    // master tx buffer
   uint8_t     mrxbuf[BUF_SIZE];    // master rx buffer

   /* Infinite loop */
   while( 1 )
   {
      // update mtxbuf
      for( i = 0; i < BUF_SIZE; i++ )
      {
         mtxbuf[i] = mtxch;

         if( mtxch == 'Z' )
         {
            mtxch = 'A';
         }
         else
         {
            mtxch++;
         }
      }

      // arm rx transfer
      HAL_DMAC_Setup( ( DMACn_Type* )DMAC1, ( uint32_t )mrxbuf, BUF_SIZE );

      // master tx: 'A' ~ 'Z'
      {
         // nSS = 0
         SPI_SS_control( ( SPIn_Type* )SPI1, 0 );

         // arm tx transfer
         HAL_DMAC_Setup( ( DMACn_Type* )DMAC0, ( uint32_t )mtxbuf, BUF_SIZE );

         // wait until tx transfer complete
         while( 1 )
         {
            // if tx transfer complete
            if( ( TransferComplete0 == TRUE ) && ( SPI1->SR_b.SPInIFLAG ) )
            {
               // clear flag
               TransferComplete0 = FALSE;

               // clear interrupt flag
               SPIn_ClearStatus( ( SPIn_Type* )SPI1, SPIn_STATUS_SPInIFLAG );

               // exit
               break;
            }
         }

         // nSS = 1
         SPI_SS_control( ( SPIn_Type* )SPI1, 1 );
      }

      // master rx
      {
         while( 1 )
         {
            // if rx transfer complete
            if( TransferComplete1 == TRUE )
            {
               // clear flag
               TransferComplete1 = FALSE;

               // exit
               break;
            }
         }
      }

      // print
      _DBG( "Master Tx Data = " );
      for( i = 0; i < BUF_SIZE; i++ )
      {
         _DBC( mtxbuf[i] );
      }
      _DBG( "\n\r" );
      _DBG( "Master Rx Data = " );
      for( i = 0; i < BUF_SIZE; i++ )
      {
         _DBC( mrxbuf[i] );
      }
      _DBG( "\n\r\n\r" );

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SPI1_Configure */
   SPI1_Configure();

   /* DMAC0_Configure */
   DMAC0_Configure();

   /* DMAC1_Configure */
   DMAC1_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DMACn_SPInRun */
   DMACn_SPInRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

