/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DMAC0_IRQHandler_IT( void );
void DMAC1_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void UART1_Configure( void );
void DMAC0_Configure( void );
void DMAC1_Configure( void );
void DMACn_UARTnRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

// buffer size
#define BUF_SIZE     4

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "DMACn_UARTn Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: DMAC0 / DMAC1 / UART1 \n\r"
                        "Counterpart:       A external device like a PC \n\r"
                        "Tx Data:           Any character \n\r"
                        "UART TXD Pin:      PC10(TXD1) \n\r"
                        "UART RXD Pin:      PC11(RXD1) \n\r"
                        "************************************************\n\r\n\r";

// transfer complete flag
volatile Bool     TransferComplete0 = FALSE;
volatile Bool     TransferComplete1 = FALSE;

// transfer error flag
volatile Bool     TransferError0 = FALSE;
volatile Bool     TransferError1 = FALSE;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC0->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete0 = TRUE;

      // reset TRCIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC0->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError0 = TRUE;

      // reset TRERIFGn
      DMAC0->IESR = DMAC0->IESR
                    &  ~DMACn_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMACn_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_IRQHandler_IT( void )
{
   // if dma transfer complete interrupt
   if( DMAC1->IESR_b.TRCIFGn == 1 )      // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferComplete1 = TRUE;

      // reset TRCIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }

   // if dma transfer error interrupt
   if( DMAC1->IESR_b.TRERIFGn == 1 )      // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
   {
      // set flag
      TransferError1 = TRUE;

      // reset TRERIFGn
      DMAC1->IESR = DMAC1->IESR
                    &  ~DMAC1_IESR_TRCIFGn_Msk        // TRCIFGn[00:00]   [0] in [0(no request) 1(request; reset by w1)]
                    |   DMAC1_IESR_TRERIFGn_Msk       // TRERIFGn[01:01]   [0] in [0(no request) 1(request; reset by w1)]
                    ;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   // UART1 configuration
   {
      // uart TXD/RXD pin configuration
      {
         // print
         _DBG( "PC10 was configured as a TXD1. \n\r" );
         _DBG( "PC11 was configured as a RXD1. \n\r\n\r" );

         // configure PC10 as a TXD1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 10, ALTERN_FUNC );     // PB0            0: ----        1: ----        2: TXD1        3: ----        4: AN10        5: ----        6: ----        7: SEG12
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 10, AFSRx_AF2 );  // PC10           0: ----        1: ----        2: TXD1        3: ----        4: SC0IN       5: ----        6: ----        7: COM6/SEG27

         // configure PC11 as a RXD1
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 11, ALTERN_FUNC );     // PB1            0: ----        1: ----        2: RXD1        3: ----        4: AN11        5: LPDE        6: ----        7: SEG11
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 11, AFSRx_AF2 );  // PC11           0: ----        1: ----        2: RXD1        3: ----        4: ----        5: LPDE        6: ----        7: COM7/SEG28
      }

      // uart configuration
      {
         // get default configuration for uart: 38400-8-N-1
         HAL_UART_ConfigStructInit( &UARTn_Config );

         // init UARTn
         HAL_UART_Init( ( UARTn_Type* )UART1, &UARTn_Config );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC0_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC0_Configure( void )
{
   // DMAC0 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC0, PERSEL_UART1Tx, DIR_MemToPeri, SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC0->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC0->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC0_IRQn, 3 );
      NVIC_EnableIRQ( DMAC0_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC0 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMAC1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMAC1_Configure( void )
{
   // DMAC1 configuration
   {
      // dma configuration
      {
         // init dmacn
         HAL_DMAC_Init( ( DMACn_Type* )DMAC1, PERSEL_UART1Rx, DIR_PeriToMem,  SIZE_8bit, ERFGSTP_Enable );

         // enable interrupt
         DMAC1->IESR_b.TRCIENn = 1;       // TRCIENn[04:04]   [0] in [0(disable transfer complete interrupt) 1(enable)]
         DMAC1->IESR_b.TRERIENn = 1;      // TRERIENn[05:05]   [0] in [0(disable transfer error interrupt) 1(enable)]
      }

      // unmask interrupt
      NVIC_SetPriority( DMAC1_IRQn, 3 );
      NVIC_EnableIRQ( DMAC1_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_DMAC1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DMACn_UARTnRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DMACn_UARTnRun( void )
{
   uint8_t     buf[BUF_SIZE];    // tx/rx buffer

   // arm rx transfer
   HAL_DMAC_Setup( ( DMACn_Type* )DMAC1, ( uint32_t )buf, BUF_SIZE );

   /* Infinite loop */
   while( 1 )
   {
      // if rx transfer complete (receive data from PC)
      if( TransferComplete1 == TRUE )
      {
         // clear flag
         TransferComplete1 = FALSE;

         // arm tx transfer (send the received data to PC)
         HAL_DMAC_Setup( ( DMACn_Type* )DMAC0, ( uint32_t )buf, BUF_SIZE );
      }

      // if tx transfer complete
      if( ( TransferComplete0 == TRUE ) && ( UART1->LSR_b.TEMT ) )
      {
         // clear flag
         TransferComplete0 = FALSE;

         // arm rx transfer
         HAL_DMAC_Setup( ( DMACn_Type* )DMAC1, ( uint32_t )buf, BUF_SIZE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* UART1_Configure */
   UART1_Configure();

   /* DMAC0_Configure */
   DMAC0_Configure();

   /* DMAC1_Configure */
   DMAC1_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DMACn_UARTnRun */
   DMACn_UARTnRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

