/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void EINT0_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void EINT_Configure( void );
void GPIO_ExtIntRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "GPIO_EINT Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: INTC \n\r"
                        "GPIO Output Pin:   PB0 ~ PB3 \n\r"
                        "EINT Input Pin:    PA0(EINT0) ~ PA3(EINT0) \n\r"
                        "************************************************\n\r\n\r";

static volatile uint8_t    FlagPA0 = 0;
static volatile uint8_t    FlagPA1 = 0;
static volatile uint8_t    FlagPA2 = 0;
static volatile uint8_t    FlagPA3 = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT0_IRQHandler_IT( void )
{
   uint32_t    status;

   // get interrupt flag
   status = HAL_INT_EIntPA_GetIntStatus();

   // if interrupt was issued by PA0
   if( status & _BIT( 0 ) )
   {
      // clear interrupt flag
      HAL_INT_EIntPA_ClearIntStatus( _BIT( 0 ) );

      // set flag
      FlagPA0 = 1;

      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
      delay();
   }

   // if interrupt was issued by PA1
   if( status & _BIT( 1 ) )
   {
      // clear interrupt flag
      HAL_INT_EIntPA_ClearIntStatus( _BIT( 1 ) );

      // set flag
      FlagPA1 = 1;

      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
      delay();
   }

   // if interrupt was issued by PA2
   if( status & _BIT( 2 ) )
   {
      // clear interrupt flag
      HAL_INT_EIntPA_ClearIntStatus( _BIT( 2 ) );

      // set flag
      FlagPA2 = 1;

      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
      delay();
   }

   // if interrupt was issued by PA3
   if( status & _BIT( 3 ) )
   {
      // clear interrupt flag
      HAL_INT_EIntPA_ClearIntStatus( _BIT( 3 ) );

      // set flag
      FlagPA3 = 1;

      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT_Configure( void )
{
   // eint input pin configuration
   {
      // PA0 configuration
      {
         // print
         _DBG( "PA0 was configured as a EINT Input Mode. \n\r" );
         _DBG( "PA0 EINT was configured as a Falling Edge Trigger. \n\r" );
         _DBG( "EINT0 was configured for a PA0 EINT. \n\r\n\r" );

         // configure PA0 as a Input Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, INPUT );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 0, PUPDx_EnablePU );
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 0, DBCLK_HCLK1024 );

         // configure PA0 as a Falling Edge Trigger
         HAL_INT_EIntPx_SetReg( PORTA, 0, ITRIGx_Edge, INTCTLx_FallingEdge );

         // configure EINT0 for PA0 EINT
         HAL_INT_EIntCfg( EINT0, CONFx_PA, 0 );
      }

      // PA1 configuration
      {
         // print
         _DBG( "PA1 was configured as a EINT Input Mode. \n\r" );
         _DBG( "PA1 EINT was configured as a Rising Edge Trigger. \n\r" );
         _DBG( "EINT0 was configured for a PA1 EINT. \n\r\n\r" );

         // configure PA1 as a Input Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, INPUT );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 1, PUPDx_EnablePU );
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 1, DBCLK_HCLK1024 );

         // configure PA1 as a Rising Edge Trigger
         HAL_INT_EIntPx_SetReg( PORTA, 1, ITRIGx_Edge, INTCTLx_RisingEdge );

         // configure EINT0 for PA1 EINT
         HAL_INT_EIntCfg( EINT0, CONFx_PA, 1 );
      }

      // PA2 configuration
      {
         // print
         _DBG( "PA2 was configured as a EINT Input Mode. \n\r" );
         _DBG( "PA2 EINT was configured as a Both Edge Trigger. \n\r" );
         _DBG( "EINT0 was configured for a PA2 EINT. \n\r\n\r" );

         // configure PA2 as a Input Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, INPUT );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 2, PUPDx_EnablePU );
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 2, DBCLK_HCLK1024 );

         // configure PA2 as a Both Edge Trigger
         HAL_INT_EIntPx_SetReg( PORTA, 2, ITRIGx_Edge, INTCTLx_BothEdge );

         // configure EINT0 for PA2 EINT
         HAL_INT_EIntCfg( EINT0, CONFx_PA, 2 );
      }

      // PA3 configuration
      {
         // print
         _DBG( "PA3 was configured as a EINT Input Mode. \n\r" );
         _DBG( "PA3 EINT was configured as a Low Level Trigger. \n\r" );
         _DBG( "EINT0 was configured for a PA3 EINT. \n\r\n\r" );

         // configure PA3 as a Input Mode
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 3, INPUT );
         HAL_GPIO_ConfigPullup( ( Pn_Type* )PA, 3, PUPDx_EnablePU );
         HAL_GPIO_SetDebouncePin( ( Pn_Type* )PA, 3, DBCLK_HCLK1024 );

         // configure PA3 as a Low Level Trigger
         HAL_INT_EIntPx_SetReg( PORTA, 3, ITRIGx_Level, INTCTLx_LowLevel );

         // configure EINT0 for PA3 EINT
         HAL_INT_EIntCfg( EINT0, CONFx_PA, 3 );
      }

      // unmask Interrupt
      NVIC_SetPriority( EINT0_IRQn, 3 );
      NVIC_EnableIRQ( EINT0_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_EINT0 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_ExtIntRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_ExtIntRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
      // check flag for PA0
      if( FlagPA0 == 1 )
      {
         // clear flag
         FlagPA0 = 0;

         // print
         _DBG( "EINT0 External Interrupt was issued by PA0. \n\r" );
      }

      // check flag for PA1
      if( FlagPA1 == 1 )
      {
         // clear flag
         FlagPA1 = 0;

         // print
         _DBG( "EINT0 External Interrupt was issued by PA1. \n\r" );
      }

      // check flag for PA2
      if( FlagPA2 == 1 )
      {
         // clear flag
         FlagPA2 = 0;

         // print
         _DBG( "EINT0 External Interrupt was issued by PA2. \n\r" );
      }

      // check flag for PA3
      if( FlagPA3 == 1 )
      {
         // clear flag
         FlagPA3 = 0;

         // print
         _DBG( "EINT0 External Interrupt was issued by PA3. \n\r" );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* EINT_Configure */
   EINT_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* GPIO_ExtIntRun */
   GPIO_ExtIntRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

