/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void SysTick_ExceptionHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void GPIO_LedBlinkyRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "GPIO_LEDBlinky Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: PCU \n\r"
                        "GPIO Output Pin:   PB0 ~ PB3 \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    msec;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         SysTick_ExceptionHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SysTick_ExceptionHandler_IT( void )
{
   if( msec > 0 )
   {
      msec--;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_LedBlinkyRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_LedBlinkyRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
      // output high
      {
         // output high
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );

         // print
         _DBG( "PB0 ~ PB3 Output is High Level. \n\r" );

         // delay
         msec = 500;
         while( msec );
      }

      // output low
      {
         // output low
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );

         // print
         _DBG( "PB0 ~ PB3 Output is Low Level. \n\r" );

         // delay
         msec = 500;
         while( msec );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   // configure SysTick to 1ms interrupt
   SysTick_Config( SystemCoreClock / 1000 );

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* GPIO_LedBlinkyRun */
   GPIO_LedBlinkyRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

