/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void RTCC_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void RTCC_Configure( void );
void RTCC_TestRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "RTCC_Test Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: RTCC \n\r"
                        "GPIO Output Pin:   PB0 ~ PB3 \n\r"
                        "RTCC Output Pin:   PB7(RTCOUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         RTCC_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void RTCC_IRQHandler_IT( void )
{
   uint32_t          status;
   static uint8_t    alternate = 1;    // toggle flag

   // get interrupt flag
   status = RTCC_InInt_GetFg();

   // if RTCC interval Interrupt
   if( status == 1 )
   {
      // clear interrupt flag
      RTCC_InInt_ClrFg();

      _DBG( "20" );
      _DBH( RTCC->YEAR );
      _DBG( "/" );
      _DBH( RTCC->MONTH );
      _DBG( "/" );
      _DBH( RTCC->DAY );
      _DBG( " " );
      _DBH( RTCC->HOUR );
      _DBG( ":" );
      _DBH( RTCC->MIN );
      _DBG( ":" );
      _DBH( RTCC->SEC );
      _DBG( "                       \n\r\n\r" );

      // toggle PB0 ~ PB3
      if( alternate == 1 )
      {
         // clear flag
         alternate = 0;

         // output high
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
      else
      {
         // set flag
         alternate = 1;

         // output low
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
         HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );

      // SXIN/SXOUT pin configuration
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, ALTERN_FUNC );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );

      HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, ALTERN_FUNC );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         RTCC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void RTCC_Configure( void )
{
   RTCC_CFG_Type   RTCC_Config;

   // RTCC configuration
   {
      // rtcc output pin configuration
      {
         // print
         _DBG( "PB7 was configured as a RTCOUT. \n\r\n\r" );

         // configure PB7 as a RTCOUT.
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF2 );
      }

      // select peripheral clock: XSOSC
      HAL_SCU_Peripheral_ClockSelection( PPCLKSR_RTCCLK, RTCCLK_XSOSC );

      // Reset RTCC peripheral. If you need RTCC reset.
      HAL_SCU_Peripheral_SetReset2( 1 << SCUCG_PPRST2_RTCRST_Pos );

      RTCC_Config.rtccIntIn = RTCC_RTIN_OncePer1sec;  // RTCC Interrupt Interval : 1s
      RTCC_Config.rtccHS24 = RTCC_24HS;                // 24-hour system
      RTCC_Config.rtccOutsel = RTCC_RTO_32K;           // RTOUT 32kHz
      HAL_RTCC_Init( &RTCC_Config );

      // init RTCC -> Set to 2019/10/22(tuesday) 23:59:55
      RTCC_SetYEAR( 0x19 );
      RTCC_SetMONTH( 0x10 );
      RTCC_SetDAY( 0x22 );
      RTCC_SetWEEK( RTCC_Tuesday );
      RTCC_SetHOUR( 0x23 );
      RTCC_SetMIN( 0x59 );
      RTCC_SetSEC( 0x55 );

      // unmask Interrupt
      NVIC_SetPriority( RTCC_IRQn, 3 );
      NVIC_EnableIRQ( RTCC_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_RTCC );

      // enable RTCC
      HAL_RTCC_Cmd( ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         RTCC_TestRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void RTCC_TestRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* RTCC_Configure */
   RTCC_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* RTCC_TestRun */
   RTCC_TestRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

