/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void TIMER40_Configure( void );
void TIMER4n_OneShotRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER4n_OneShot Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER40 \n\r"
                        "Timer Output Pin:  PA0/PA1(T40OUTA/T40OUTB) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER40_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER40_Configure( void )
{
   TIMER4n_PERIODICCFG_Type   TIMER4n_Config;

   // TIMER40 configuration
   {
      // timer output pin configuration
      {
         // print
         _DBG( "PA0 was configured as a T40OUTA. \n\r" );
         _DBG( "PA1 was configured as a T40OUTB. \n\r\n\r" );

         // configure PA0 as a T40OUTA
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 0, AFSRx_AF0 );

         // configure PA1 as a T40OUTB
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, AFSRx_AF0 );
      }

      // timer one shot mode configuration
      {
         TIMER4n_Config.CkSel = TIMER4n_PCLK;               // PCLK = 32MHz
         TIMER4n_Config.Prescaler = ( 32 - 1 );             // fTIMER = PCLK / 32 = 1MHz
         TIMER4n_Config.PDR = ( 2000 - 1 );                 // Period  = PDR / fTIMER = 2000us
         TIMER4n_Config.ADR = ( 1000 );                     // Duty(A) = ADR / fTIMER = 1000us
         TIMER4n_Config.BDR = ( 500 );                      // Duty(B) = BDR / fTIMER =  500us
         TIMER4n_Config.OutBStartLevel = TIMER4n_OUTB_START_HIGH;
         TIMER4n_Config.OutAStartLevel = TIMER4n_OUTA_START_HIGH;
         TIMER4n_Config.OutBEnable = TIMER4n_OUTB_ENABLE;
         TIMER4n_Config.OutAEnable = TIMER4n_OUTA_ENABLE;
         TIMER4n_Config.ECE = TIMER4n_FALLING_EGDE;         // dummy parameter

         HAL_TIMER4n_Init( ( TIMER4n_Type* )TIMER40, TIMER4n_ONESHOT_MODE, &TIMER4n_Config );
      }
   }

   // enable timer (clear & start counter)
   HAL_TIMER4n_Cmd( ( TIMER4n_Type* )TIMER40, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER4n_OneShotRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER4n_OneShotRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* TIMER40_Configure */
   TIMER40_Configure();

   /* TIMER4n_OneShotRun */
   TIMER4n_OneShotRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

