/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER50_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER50_Configure( void );
void TIMER5n_MatchInterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER5n_MatchInterrupt Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER50 \n\r"
                        "GPIO Output Pin:   PB0 ~ PB3 \n\r"
                        "Timer Output Pin:  PF1(T50OUT) \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER50_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER50_IRQHandler_IT( void )
{
   static uint32_t   count = 0;
   static uint8_t    alternate = 1;    // toggle flag

   // if Timer/Counter 2n Match Interrupt
   if( ( HAL_TIMER5n_GetStatus( ( TIMER5n_Type* )TIMER50 ) & TIMER5n_CR_T5nMIFLAG_Msk ) == TIMER5n_CR_T5nMIFLAG_Msk )
   {
      // clear interrupt flag
      HAL_TIMER5n_ClearStatus( ( TIMER5n_Type* )TIMER50, TIMER5n_CR_T5nMIFLAG_Msk );

      // indicate
      if( ( count++ % 1000 ) == 0 )
      {
         // toggle PB0 ~ PB3
         if( alternate == 1 )
         {
            // clear flag
            alternate = 0;

            // output high
            HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 0 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 1 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 2 ) );
            HAL_GPIO_SetPin( ( Pn_Type* )PB, _BIT( 3 ) );
         }
         else
         {
            // set flag
            alternate = 1;

            // output low
            HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );
            HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // gpio output pin configurtion
   {
      // print
      _DBG( "PB0 ~ PB3 was configured as a GPIO Output Mode. \n\r\n\r" );

      // configure PB0 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 0 ) );

      // configure PB1 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 1 ) );

      // configure PB2 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 2, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 2 ) );

      // configure PB3 as a Output Mode
      HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );
      HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 3, PUPDx_Disable );
      HAL_GPIO_ClearPin( ( Pn_Type* )PB, _BIT( 3 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER50_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER50_Configure( void )
{
   TIMER5n_CFG_Type     TIMER5n_Config;

   // TIMER50 configuration
   {
      // timer output pin configuration
      {
         // print
         _DBG( "PF1 was configured as a T50OUT. \n\r\n\r" );

         // configure PF1 as a T50OUT
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PF, 1, ALTERN_FUNC );      // PB2            0: T50OUT      1: ----        2: ----        3: ----        4: ----        5: ----        6: ----        7: SEG10
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PF, 1, AFSRx_AF0 );   // PF1            0: T50OUT      1: ----        2: ----        3: ----        4: AN13        5: ----        6: ----        7: SEG22
      }

      // timer match mode configuration
      {
         // select peripheral clock: PCLK
         HAL_SCU_Peripheral_ClockSelection( PPCLKSR_T50CLK, T50CLK_PCLK );

         // init timer
         TIMER5n_Config.T5nMS = TIMER5n_CR_T5nMS_IntervalMode;
         TIMER5n_Config.T5nCLK = TIMER5n_CR_T5nCLK_IntPrescaledClock;   // PCLK = 32MHz
         TIMER5n_Config.T5nECE = TIMER5n_CR_T5nECE_FallingEdge;         // dummy parameter
         TIMER5n_Config.T5nINSEL = TIMER5n_CR_T5nINSEL_ExtClock;        // dummy parameter
         TIMER5n_Config.T5nINPOL = TIMER5n_CR_T5nINPOL_FallingEdge;     // dummy parameter
         TIMER5n_Config.T5nOPOL = TIMER5n_CR_T5nOPOL_StartLow;
         TIMER5n_Config.ADR = 1000 - 1;                                 // Period  = ADR / fTIMER = 1000us
         TIMER5n_Config.BDR = 0;                                        // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
         TIMER5n_Config.Prescaler = 32 - 1;                             // fTIMER = PCLK / 32 = 1MHz
         HAL_TIMER5n_Init( ( TIMER5n_Type* )TIMER50, &TIMER5n_Config );
         HAL_TIMER5n_ConfigInterrupt( ( TIMER5n_Type* )TIMER50, TIMER5n_CR_MATCH_INTR, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( TIMER50_IRQn, 3 );
      NVIC_EnableIRQ( TIMER50_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_TIMER50 );

      // enable timer (clear & start counter)
      HAL_TIMER5n_Cmd( ( TIMER5n_Type* )TIMER50, ENABLE );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER5n_MatchInterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER5n_MatchInterruptRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER50_Configure */
   TIMER50_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER5n_MatchInterruptRun */
   TIMER5n_MatchInterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

