/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void SPI_SS_control( USART1n_Type* USART1x, char ch );
void USART10_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SPI10_Configure( void );
void USART1n_SPI_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_SPI_Interrupt Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:   SPI10 Master \n\r"
                        "Counterpart:         Self (Loopback) \n\r"
                        "Master Tx Data:      'A' ~ 'Z' \n\r"
                        "SPI Master MOSI Pin: PA2(MOSI10)      \n\r"
                        "SPI Master MISO Pin: PA3(MISO10)      \n\r"
                        "SPI Master SCLK Pin: PA4(SCK10)       \n\r"
                        "SPI Master nSS Pin:  PA1(Output Mode) \n\r"
                        "Prerequisite:        Connect PA2(MOSI10) and PA3(MISO10) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

volatile uint8_t     FlagRXC = 0;
volatile uint8_t     mrxch;         // master rx character

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI_SS_control
 * @param[in]     USART1x
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI_SS_control( USART1n_Type* USART1x, char ch )
{
#if 1    // supported
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      if( ch == 0 )
      {
         PA->OUTDR_b.OUTDR1 = 0;    // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
      }                             // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
      else
      {
         PA->OUTDR_b.OUTDR1 = 1;    // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
      }                             // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
   }
#endif

#if 0    // not supported
   if( USART1x == ( USART1n_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
#endif

#if 0    // not supported
   if( USART1x == ( USART1n_Type* )USART12 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR15 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR15 = 1;
      }
   }
#endif

#if 0    // not supported
   if( USART1x == ( USART1n_Type* )USART13 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR11 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR11 = 1;
      }
   }
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART10_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   // get interrupt flag
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART10 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // if Receive Complete Interrupt
   if( USART10->CR1_b.RXCIEn == 1 && ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      // set flag
      FlagRXC = 1;

      // get byte
      mrxch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART10 );
   }

   // if Transmit Complete Interrupt
   if( USART10->CR1_b.TXCIEn == 1 && ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      // clear interrupt flag
      HAL_USART_ClearStatus( ( USART1n_Type* )USART10, USART1n_STATUS_TXC );

      // nss = 1
      SPI_SS_control( ( USART1n_Type* )USART10, 1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // SPI10 configuration
   {
      // spi MOSI/MISO/SCLK/nSS pin configuration
      {
         // print
         _DBG( "PA2 was configured as a MOSI10. \n\r" );
         _DBG( "PA3 was configured as a MISO10. \n\r" );
         _DBG( "PA4 was configured as a SCK10. \n\r" );
#if 1    // if Msater Mode
         _DBG( "PA1 was configured as a Output Mode. \n\r\n\r" );
#else    // if Slave Mode
         _DBG( "PA1 was configured as a SS10. \n\r\n\r" );
#endif

         // configure PA2 as a MOSI10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, ALTERN_FUNC );      // PA2            0: T41OUTA     1: T41INP      2: TXD10       3: MOSI10      4: AN2         5: CP1N0       6: CP1OUT      7: SEG18
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 2, AFSRx_AF3 );   // PC6   SWCLK    0: SWCLK       1: ----        2: TXD10       3: MOSI10      4: SC0DATA     5: SC0TXD      6: ----        7: ----

         // configure PA3 as a MISO10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 3, ALTERN_FUNC );      // PA3            0: T41OUTB     1: EC41        2: RXD10       3: MISO10      4: AN3         5: CP1P0       6: ADTRG       7: SEG17
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 3, AFSRx_AF3 );   // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24

         // configure PA4 as a SCK10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, ALTERN_FUNC );      // PA4            0: ----        1: T43INP      2: SS1         3: SCK10       4: AN4         5: CP0N1       6: CP1N1       7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 4, AFSRx_AF3 );   // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25

#if 1    // if Msater Mode
         // configure PA1 as a Output Mode                        // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, PUSH_PULL_OUTPUT ); // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
#else    // if Slave Mode
         // configure PA1 as a SS10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 1, ALTERN_FUNC );      // PA1            0: T40OUTB     1: EC40        2: T42INP      3: SS10        4: AN1         5: CP0P0       6: ----        7: SEG19
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 1, AFSRx_AF3 );   // PC9            0: ----        1: ----        2: LPRXD       3: SS10        4: SC0PWR      5: ----        6: ----        7: COM5/SEG26
#endif
      }

      // spi configuration
      {
         // get default configuration for spi: 38400
         HAL_USART_SPI_Mode_Config( &USART1n_Config );

         // init USART1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

#if 1    // if Msater Mode
         // enable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

         // nss = 1
         SPI_SS_control( ( USART1n_Type* )USART10, 1 );
#else    // if Slave Mode
         // disable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, DISABLE );

         // enable slave nss pin function
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_USTSSEN, ENABLE );
#endif

         // enable interrupt
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_TXC, ENABLE );
         HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_RXC, ENABLE );

         // enable USART1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }

      // unmask Interrupt
      NVIC_SetPriority( USART10_IRQn, 3 );
      NVIC_EnableIRQ( USART10_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_USART10 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_SPI_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_SPI_InterruptRun( void )
{
   uint8_t     mtxch = 'A';   // master tx character

   /* Infinite loop */
   while( 1 )
   {
      // master tx: 'A' ~ 'Z'
      SPI_SS_control( ( USART1n_Type* )USART10, 0 );
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &mtxch, 1, NONE_BLOCKING );

      // master rx
      while( 1 )
      {
         if( FlagRXC == 1 )
         {
            // clear flag
            FlagRXC = 0;

            // exit
            break;
         }
      }

      // print
      _DBG( "Master Tx Data = " );
      _DBC( mtxch );
      _DBG( "\n\r" );
      _DBG( "Master Rx Data = " );
      _DBC( mrxch );
      _DBG( "\n\r\n\r" );

      // mtxch++
      if( mtxch == 'Z' )
      {
         mtxch = 'A';
      }
      else
      {
         mtxch++;
      }

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SPI10_Configure */
   SPI10_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_SPI_InterruptRun */
   USART1n_SPI_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

