/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31L12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void USRT10_Configure( void );
void USART1n_USRT_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_USRT_Polling Example \n\r"
                        "  - MCU:        A31L123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:     USRT10 Master \n\r"
                        "Counterpart:           Self (Loopback) \n\r"
                        "Master Tx Data:        'A' ~ 'Z' \n\r"
                        "USRT Master TXD Pin:   PA2(TXD10) \n\r"
                        "USRT Master RXD Pin:   PA3(RXD10) \n\r"
                        "USRT Master SCK Pin:   PA4(SCK10) \n\r"
                        "Prerequisite:          Connect PA2(TXD10) and PA3(RXD10) with a jumper. \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // USRT10 configuration
   {
      // usrt TXD/RXD/SCK pin configuration
      {
         // print
         _DBG( "PA2 was configured as a TXD10. \n\r" );
         _DBG( "PA3 was configured as a RXD10. \n\r" );
         _DBG( "PA4 was configured as a SCK10. \n\r\n\r" );

         // configure PA2 as a TXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 2, ALTERN_FUNC );    // PA2            0: T41OUTA     1: T41INP      2: TXD10       3: MOSI10      4: AN2         5: CP1N0       6: CP1OUT      7: SEG18
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 2, AFSRx_AF2 );    // PC6   SWCLK    0: SWCLK       1: ----        2: TXD10       3: MOSI10      4: SC0DATA     5: SC0TXD      6: ----        7: ----

         // configure PA3 as a RXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 3, ALTERN_FUNC );    // PA3            0: T41OUTB     1: EC41        2: RXD10       3: MISO10      4: AN3         5: CP1P0       6: ADTRG       7: SEG17
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 3, AFSRx_AF2 );    // PC7            0: T40OUTA     1: T40INP      2: RXD10       3: MISO10      4: SC0RST      5: SC0RXD      6: SS1         7: SEG24

         // configure PA4 as a SCK10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PA, 4, ALTERN_FUNC );    // PA4            0: ----        1: T43INP      2: SS1         3: SCK10       4: AN4         5: CP0N1       6: CP1N1       7: ----
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PA, 4, AFSRx_AF3 );    // PC8            0: ----        1: ----        2: LPTXD       3: SCK10       4: SC0CLK      5: ----        6: ----        7: COM4/SEG25
      }

      // usrt configuration
      {
         // get default configuration for usrt: 38400-8-N-1
         HAL_USART_USRT_Mode_Config( &USART1n_Config );

         // init USART1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

#if 1    // if Msater Mode
         // enable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

         // select clock waveform
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_DISSCK, DISABLE );
#else    // if Slave Mode
         // disable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, DISABLE );
#endif

         // enable USART1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_USRT_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_USRT_PollingRun( void )
{
   uint8_t     mtxch = 'A';      // master tx character
   uint8_t     mrxch;            // master rx character

   /* Infinite loop */
   while( 1 )
   {
      // master tx: 'A' ~ 'Z'
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &mtxch, 1, BLOCKING );

      // master rx
      HAL_USART_Receive( ( USART1n_Type* )USART10, &mrxch, 1, BLOCKING );

      // print
      _DBG( "Master Tx Data = " );
      _DBC( mtxch );
      _DBG( "\n\r" );
      _DBG( "Master Rx Data = " );
      _DBC( mrxch );
      _DBG( "\n\r\n\r" );

      // mtxch++
      if( mtxch == 'Z' )
      {
         mtxch = 'A';
      }
      else
      {
         mtxch++;
      }

      // delay
      delay();
      delay();
      delay();
      delay();
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* USRT10_Configure */
   USRT10_Configure();

   /* USART1n_USRT_PollingRun */
   USART1n_USRT_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 32MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

